/*
 *  Copyright (C) 2002-23 - ntop.org
 *
 *  http://www.ntop.org/
 *
 * This code is proprietary code subject to the terms and conditions
 * defined in LICENSE file which is part of this source code package.
 */

#ifndef _PINFO_H_
#define _PINFO_H_

/*
  For some unknown reasons, this is a work around to avoid linking issues
 */
static struct processFlowPacketInfo* parseFlowPacket(struct processFlowPacketInfo *pinfo,
						     int packet_if_idx /* -1 = unknown */,
						     as_info *as,
						     u_int8_t rx_packet, /* 1=RX, 0=TX */
						     u_int32_t subflow_id, u_int8_t proto, u_int16_t numFragments,
						     u_int16_t ip_offset, u_int8_t sampledPacket,
						     u_int32_t numPkts, u_int8_t tos, u_int16_t ip_id, u_int8_t ttl,
						     u_int16_t vlanId_src,  u_int16_t vlanId_dst,
						     u_int16_t innerVlanId, u_int16_t outerVlanId,
						     u_int32_t tunnel_id,
						     u_int16_t session_id, u_int16_t gtp_offset,
						     struct eth_header *ehdr,
						     IpAddress *src, u_int16_t sport,
						     IpAddress *dst, u_int16_t dport,
						     u_int8_t untunneled_proto,
						     IpAddress *untunneled_src, u_int16_t untunneled_sport,
						     IpAddress *untunneled_dst, u_int16_t untunneled_dport,
						     u_int16_t ndpi_proto_major,
						     u_int len, u_int16_t tcpWin, u_int8_t tcpFlags,
						     u_int32_t tcpSeqNum, u_int32_t tcpAckNum, u_int16_t tcpLen,
						     u_int16_t tcpMaxSegmentSize, u_int8_t tcpWinScale,
						     u_int8_t icmpType, u_int8_t icmpCode,
						     u_int16_t numMplsLabels,
						     u_char *mplsLabels /* [MAX_NUM_MPLS_LABELS][MPLS_LABEL_LEN] */,
						     u_int32_t if_input, u_int32_t if_output,
						     struct pcap_pkthdr *h, u_char *p,
						     u_char *tcp_udp, u_char *payload,
						     u_int16_t payload_shift, u_int payloadLen,
						     u_int originalPayloadLen,
						     const struct timeval * const _firstSeen, /* Always set to 0 unless numPkts > 0 */
						     u_int16_t src_mask, u_int16_t dst_mask,
						     u_int32_t flow_sender_ip,
						     struct in6_addr *flow_sender_ip6,
						     u_int32_t packet_hash,
						     u_int8_t engine_type, u_int8_t engine_id,
						     char *osi_src, char *osi_dst,
						     struct generic_netflow_record *record,
						     flow_marker *verdict,
						     flow_source source) {
    pinfo->packet_if_idx = packet_if_idx,
    pinfo->rx_packet = rx_packet,
    pinfo->subflow_id = subflow_id,
    pinfo->proto = proto,
    pinfo->numFragments = numFragments,
    pinfo->ip_offset = ip_offset,
    pinfo->sampledPacket = sampledPacket,
    pinfo->numPkts = numPkts,
    pinfo->tos = tos,
    pinfo->ip_id = ip_id,
    pinfo->ttl = ttl,
    pinfo->vlanId_src  = readOnlyGlobals.ignoreInnerVlanId ? 0 : vlanId_src,
    pinfo->vlanId_dst  = readOnlyGlobals.ignoreOuterVlanId ? 0 : vlanId_dst,
    pinfo->innerVlanId = readOnlyGlobals.ignoreInnerVlanId ? 0 : innerVlanId,
    pinfo->outerVlanId = readOnlyGlobals.ignoreOuterVlanId ? 0 : outerVlanId,
    pinfo->tunnel_id = tunnel_id,
    pinfo->session_id = session_id,
    pinfo->gtp_offset = gtp_offset,
    pinfo->ehdr = ehdr,
    pinfo->src = src,
    pinfo->sport = sport,
    pinfo->dst = dst,
    pinfo->dport = dport,
    pinfo->untunneled_proto = untunneled_proto,
    pinfo->untunneled_src = untunneled_src,
    pinfo->untunneled_sport = untunneled_sport,
    pinfo->untunneled_dst = untunneled_dst,
    pinfo->untunneled_dport = untunneled_dport,
    pinfo->ndpi_proto_major = ndpi_proto_major,
    pinfo->len = len,
    pinfo->tcpWin = tcpWin,
    pinfo->tcpFlags = tcpFlags,
    pinfo->tcpSeqNum = tcpSeqNum,
    pinfo->tcpAckNum = tcpAckNum,
    pinfo->tcpLen = tcpLen,
    pinfo->tcpMaxSegmentSize = tcpMaxSegmentSize,
    pinfo->tcpWinScale = tcpWinScale,
    pinfo->icmpType = icmpType,
    pinfo->icmpCode = icmpCode,
    pinfo->numMplsLabels = numMplsLabels,
    pinfo->mplsLabels = mplsLabels,
    pinfo->if_input = if_input,
    pinfo->if_output = if_output,
    pinfo->h = h,
    pinfo->p = p,
    pinfo->tcp_udp = tcp_udp,
    pinfo->payload = payload,
    pinfo->payload_shift = payload_shift,
    pinfo->payloadLen = payloadLen,
    pinfo->originalPayloadLen = originalPayloadLen,
    pinfo->_firstSeen = _firstSeen,
    pinfo->src_mask = src_mask,
    pinfo->dst_mask = dst_mask,
    pinfo->flow_sender_ip = flow_sender_ip,
    pinfo->flow_sender_ip6 = flow_sender_ip6,
    pinfo->packet_hash = packet_hash,
    pinfo->engine_type = engine_type,
    pinfo->engine_id = engine_id,
    pinfo->osi_src = osi_src,
    pinfo->osi_dst = osi_dst,
    pinfo->record = record,
    pinfo->verdict = verdict,
    pinfo->source_nature  = source;
    
  if(as)
    memcpy(&pinfo->as, as, sizeof(as_info));
  else
    memset(&pinfo->as, 0,  sizeof(as_info));

  /* Payload len validation */
  pinfo->payloadLen = ndpi_min(pinfo->payloadLen, (h->caplen - payload_shift));
  
  return(pinfo);
};

#endif /* _PINFO_H_ */
